(function () {
    /**
     * Représente les projets.
     *
     * @param {Object} data
     *
     * @constructor
     */
    function Projet(data) {
        this.idProjetGnn = null;
        this.nom = '';
        this.type = '';
        this.geo = '';
        this.place = '';
        this.proprietaire = '';
        this.isSync = false;
        this.sections = [];
        this.dateCreation = new Date();

        this.depotsAutoincrement = 1;
        var depots = [];
        Object.defineProperty(this, 'depots', {
            set: function (newDepots) {
                if (!(newDepots instanceof Array)) {
                    throw Error("'depots' doit être un Array");
                }
                for (var i = 0; i < newDepots.length; i++) {
                    var depot = new Depot();
                    Object.assign(depot, newDepots[i]);
                    depots.push(depot);
                }
            },
            get: function () {
                return depots;
            },
            enumerable: true
        });

        this.populateFromStorage(data);
    }
    Projet.prototype = Object.create(GeneanetIndexedDBEntity.prototype);
    Projet.prototype.constructor = Projet;
    Projet.objectStoreName = 'projet';
    Projet.prototype.getObjectStoreName = function () {
        return Projet.objectStoreName;
    };

    /**
     * Compte le nombre de photos dans le projet.
     *
     * @param {integer} onlyNotSync Si true : permet de limiter aux photos non publiées.
     *
     * @return {integer}
     */
    Projet.prototype.countPhotos = function (onlyNotSync) {
        var nbPhotos = 0;
        for (var i = 0; i < this.depots.length; i++) {
            nbPhotos += this.depots[i].countPhotos(onlyNotSync);
        }

        return nbPhotos;
    };

    /**
     * Crée et ajoute un depot au cimetière.
     *
     * @return {Depot}
     */
    Projet.prototype.addDepot = function () {
        var depot = new Depot();
        depot.id = this.depotsAutoincrement;
        this.depotsAutoincrement++;

        this.depots.push(depot);

        return depot;
    };

    /**
     * Supprime un depot du projet.
     *
     * @param {Depot} depot
     */
    Projet.prototype.deleteDepot = function (depot) {
        var position = this.depots.indexOf(depot);
        if (position != -1) {
            this.depots.splice(position, 1);
        }
    };

    /**
     * Supprime les depots vides dans le projet.
     *
     * @return {boolean} true si des dépôts ont été supprimés, false si rien supprimé.
     */
    Projet.prototype.deleteEmptyDepots = function () {
        var depots = this.depots.slice(0);
        for (var i = 0; i < depots.length; i++) {
            if (depots[i].photos.length == 0) {
                this.deleteDepot(depots[i]);
            }
        }

        return this.depots.length != depots.length;
    };

    /**
     * Retourne une partie des depots du projet.
     *
     * @param {integer} from
     * @param {integer} limit
     * @param {boolean} reverse
     *
     * @return {Array}
     */
    Projet.prototype.getDepotsGroup = function (from, limit, reverse) {
        if (!reverse) {
            return this.depots.slice(from, from+limit);
        } else {
            var depots = [];
            from = this.depots.length - from;
            limit = from - limit;
            limit = Math.max(limit, 0);
            for (var i = from; i > limit; i--) {
                depots.push(this.depots[i - 1]);
            }

            return depots;
        }
    };

    /**
     * Retourne le depot ayant l'id renseigné.
     *
     * @param {integer} id
     *
     * @return {Depot|null}
     */
    Projet.prototype.findDepotById = function (id) {
        for (var i = 0; i < this.depots.length; i++) {
            if (this.depots[i].id == id) {
                return this.depots[i];
            }
        }

        return null;
    };

    /**
     * Permet de renseigner le champs geo depuis plusieurs infos.
     *
     * @param {Array} elementsGeo
     */
    Projet.prototype.setGeo = function (elementsGeo) {
        this.geo = elementsGeo.join(",");
    };

    /**
     * Permet de récupérer le champs geo avec les éléments séparés.
     *
     * @return {Array}
     */
    Projet.prototype.getGeo = function () {
        return this.geo.split(',');
    };

    /**
     * Indique si le projet est privé ou pas.
     *
     * @return {boolean}
     */
    Projet.prototype.isPrivate = function () {
        return this.proprietaire.trim().length > 0 || this.idProjetGnn == null;
    };

    /**
     * Retourne la première url d'image dans ce projet.
     *
     * @return {string|null}
     */
    Projet.prototype.getFirstPhotoUrl = function () {
        return this.depots[0] ? this.depots[0].getFirstPhotoUrl() : null;
    };

    /**
     * Indique si le projet a des photos qui ont besoin d'être synchronisées.
     *
     * @return {boolean}
     */
    Projet.prototype.hasPhotosToSynchro = function () {
        for (var i = 0; i < this.depots.length; i++) {
            if (this.depots[i].hasPhotosToSynchro()) {
                return true;
            }
        }

        return false;
    };

    /**
     * Retourne les photos qui ont besoin d'être synchronisées.
     *
     * @return {Array}
     */
    Projet.prototype.getPhotosToSynchro = function () {
        var photosToSynchro = [];

        for (var i = 0; i < this.depots.length; i++) {
            var depot = this.depots[i];

            for (var j = 0; j < depot.photos.length; j++) {
                var photo = depot.photos[j];

                if (!photo.isSync) {
                    photosToSynchro.push({projet: this, depot: depot, photo: photo});
                }
            }
        }

        return photosToSynchro;
    };

    /**
     * Représente un depot, stocké directement avec son projet.
     *
     * @constructor
     */
    function Depot() {
        this.id = null;
        this.base64Portrait = null;
        this.base64Landscape = null;
        this.sectionId = null;

        this.photosAutoincrement = 1;
        var photos = [];
        Object.defineProperty(this, 'photos', {
            set: function (newPhotos) {
                if (!(newPhotos instanceof Array)) {
                    throw Error("'photos' doit être un Array");
                }
                for (var i = 0; i < newPhotos.length; i++) {
                    var photo = new Photo();
                    Object.assign(photo, newPhotos[i]);
                    photos.push(photo);
                }
            },
            get: function () {
                return photos;
            },
            enumerable: true
        });
    }

    /**
     * Crée et ajoute une photo au depot.
     *
     * @param {Photo} photo Permet de préciser une photo déjà existante.
     *
     * @return {Photo}
     */
    Depot.prototype.addPhoto = function (photo) {
        if (!photo) {
            photo = new Photo();
        }
        photo.id = this.photosAutoincrement;
        this.photosAutoincrement++;

        this.photos.push(photo);

        return photo;
    };

    /**
     * Supprime une photo d'un depot.
     *
     * @param {Photo} photo
     */
    Depot.prototype.deletePhoto = function (photo) {
        var position = this.photos.indexOf(photo);
        if (position != -1) {
            this.photos.splice(position, 1);
        }
    };

    /**
     * Compte le nombre de photos dans le depot.
     *
     * @param {integer} onlyNotSync Si true : permet de limiter aux photos non publiées.
     *
     * @return {integer}
     */
    Depot.prototype.countPhotos = function (onlyNotSync) {
        if (onlyNotSync) {
            var nbPhotos = 0;
            for (var i = 0; i < this.photos.length; i++) {
                if (!this.photos[i].isSync) {
                    nbPhotos++;
                }
            }

            return nbPhotos;
        }

        return this.photos.length;
    };

    /**
     * Retourne la première url d'image dans ce depot.
     *
     * @return {string|null}
     */
    Depot.prototype.getFirstPhotoUrl = function () {
        return this.photos[0] ? this.photos[0].nativeUrl : null;
    };

    /**
     * Remet à 0 les bases 64 générés.
     */
    Depot.prototype.resetBase64 = function () {
        this.base64Portrait = null;
        this.base64Landscape = null;
    };

    /**
     * Indique si le depot a des photos qui ont besoin d'être synchronisées.
     *
     * @return {boolean}
     */
    Depot.prototype.hasPhotosToSynchro = function () {
        for (var i = 0; i < this.photos.length; i++) {
            if (!this.photos[i].isSync) {
                return true;
            }
        }

        return false;
    };

    /**
     * Représente une photo, socké directement avec son depot.
     *
     * @constructor
     */
    function Photo() {
        this.id = null;
        this.nativeUrl = '';
        this.isSync = false;
    }

    window.Projet = window.Projet || Projet;
})();
