'use strict';

/**
 * Service spécifique à l'application Sauvons nos tombes
 * Permet de manipuler les données dans IndexedDB.
 **/
var indexedDBService = (function () {
    /**
     * Service gérant la connexion et les requêtes.
     *
     * @constructor
     */
    function IndexedDBService() {
        this.DB_NAME = 'SauvonsNosTombes';
        this.DB_VERSION = 1;
        this.storage = null;

        // Contient la Promise liée à l'init.
        this.readyPromise = null;
    }

    /**
     * Initialise le service d'accès au stockage.
     *
     * @return {Promise}
     */
    IndexedDBService.prototype.init = function () {
        if (this.readyPromise) {
            // Init déjà effectué.
            return this.readyPromise;
        }

        var storage = new GeneanetIndexedDB(this.DB_NAME, this.DB_VERSION);

        // On enregistre les entitées utilisées.
        storage.registerEntity(
            Projet,
            Projet.objectStoreName,
            {keyPath: 'id', autoIncrement: true}
        );

        var that = this;
        this.readyPromise = storage.initConnexion().then(function () {
            that.storage = storage;
        });

        return this.readyPromise;
    };

    /**
     * Permet de récupérer un projet depuis son id.
     *
     * @param {Integer} id
     *
     * @return {Promise}
     */
    IndexedDBService.prototype.getProjetFromId = function (id) {
        id = parseInt(id);
        var that = this;

        return new Promise(function (resolve, reject) {
            that.storage.getByIds(Projet.objectStoreName, [id]).then(function (projets) {
                resolve(projets[id]);
            }).catch(function (e) {
                reject(e);
            });
        });
    };

    /**
     * Permet de récupérer tous les projets dans l'ordre desc de création.
     *
     * @return {Promise}
     */
    IndexedDBService.prototype.getAllProjets = function () {
        return this.storage.getAll(Projet.objectStoreName, GeneanetIndexedDB.CURSOR_DIRECTION.PREV);
    };

    /**
     * Permet d'insérer un ou des projets.
     *
     * @param {Projet|Array} projet
     *
     * @return {Promise}
     */
    IndexedDBService.prototype.insertProjet = function (projet) {
        return this.storage.insert(projet);
    };

    /**
     * Permet de mettre à jour un projet.
     *
     * @param {Projet} projet
     *
     * @return {Promise}
     */
    IndexedDBService.prototype.updateProjet = function (projet) {
        return this.storage.update(projet);
    };

    /**
     * Permet de supprimer un projet via son id.
     *
     * @param {Integer} id
     *
     * @return {Promise}
     */
    IndexedDBService.prototype.deleteProjetFromId = function (id) {
        var configDelete = {};
        configDelete[Projet.objectStoreName] = [id];

        return this.storage.delete(configDelete);
    };

    return [
        IndexedDBService
    ];
})();
