'use strict';

/**
 * Service spécifique à l'application Sauvons nos tombes
 * Permet de gérer des migrations.
 **/
var migrationService = (function () {
    /**
     * Service gérant les migrations de données.
     *
     * @param {Object} GeneanetLocalStorage
     * @param {Object} IndexedDBService
     * @param {Object} GeneanetDatabase
     * @param {Object} ConfigurationService
     *
     * @constructor
     */
    function MigrationService(GeneanetLocalStorage, IndexedDBService, GeneanetDatabase, ConfigurationService) {
        this.GeneanetLocalStorage = GeneanetLocalStorage;
        this.IndexedDBService = IndexedDBService;
        this.GeneanetDatabase = GeneanetDatabase;
        this.ConfigurationService = ConfigurationService;
    }

    /**
     * Indique si la migration de WebSQL vers indexedDB est faite.
     *
     * @return {Boolean}
     */
    MigrationService.prototype.webSqlToIndexedDBIsMigrated = function () {
        return this.GeneanetLocalStorage.getItemWithDefaultValue('webSqlToIndexedDB', false, 'migration');
    };

    /**
     * Exécute les migrations nécessaires.
     *
     * @return {Promise}
     */
    MigrationService.prototype.migrate = function () {
        var that = this;

        return new Promise(function (resolve, reject) {
            that.IndexedDBService.init().then(function () {
                // On regarde si la migration webSQL -> IndexedDB a été effectuée.
                var webSqlToIndexedDB = that.webSqlToIndexedDBIsMigrated();

                if (!webSqlToIndexedDB) {
                    // On regarde si l'appli a bien les tables.
                    var request = that.GeneanetDatabase.createRequest("SELECT * FROM sqlite_master WHERE name='projet'");
                    that.GeneanetDatabase.executeRequest(request, function(tx, results) {
                        if (results.rows.length > 0) {
                            console.log("les tables existes, on va lancer la migration.");

                            // On va récupérer toutes les infos de webSQL.
                            var sql = "SELECT p.id_projet_local as id_projet_local, p.id_projet_gnn as id_projet_gnn, p.nom_projet as nom_projet, p.place as place, p.type_projet as type_projet, p.date_creation as date_creation, p.is_sync as is_sync, p.geo as geo, p.proprietaire as proprietaire, d.id_depot as id_depot, d.lieu_depot as lieu_depot, d.base64_portrait as base64_portrait, d.base64_landscape as base64_landscape, picture.id_picture as id_picture, picture.native_url as native_url, picture.is_sync as picture_is_sync FROM projet as p";
                            sql += " LEFT JOIN depot as d ON p.id_projet_local = d.id_projet_local";
                            sql += " LEFT JOIN photo as picture ON d.id_depot = picture.id_depot";
                            var request = that.GeneanetDatabase.createRequest(sql);

                            that.GeneanetDatabase.executeRequest(request, function(tx, results) {
                                var projets = {};
                                var depots = {};
                                for (var i = 0; i < results.rows.length; i++) {
                                    var result = results.rows[i];
                                    var idProjet = result.id_projet_local;
                                    var idDepot = result.id_depot;
                                    var idPhoto = result.id_picture;

                                    // On crée le projet s'il n'existe pas déjà.
                                    var projet;
                                    if (!projets[idProjet]) {
                                        projet = new Projet();
                                        projet.id = idProjet;
                                        projet.idProjetGnn = result.id_projet_gnn;
                                        projet.nom = result.nom_projet;
                                        projet.type = result.type_projet;
                                        projet.geo = result.geo;
                                        projet.place = result.place;
                                        projet.proprietaire = result.proprietaire == null ? '' : result.proprietaire;
                                        projet.isSync = result.is_sync == "true" ? true : false;
                                        projet.dateCreation = new Date(result.date_creation);
                                        projet.depotsAutoincrement = new Date(result.date_creation);

                                        projets[idProjet] = {projet: projet, depotsAutoincrementMax: 1};
                                    } else {
                                        projet = projets[idProjet].projet;
                                    }

                                    // On regarde si on a un depot et s'il est déjà créé.
                                    if (idDepot) {
                                        var depot;
                                        if (!depots[idDepot]) {
                                            depot = projet.addDepot();
                                            depot.id = idDepot;
                                            depot.base64Portrait = result.base64_portrait;
                                            depot.base64Landscape = result.base64_landscape;

                                            depots[idDepot] = {depot: depot, photosAutoincrementMax: 1};

                                            projets[idProjet].depotsAutoincrementMax = Math.max(projets[idProjet].depotsAutoincrementMax, idDepot);
                                        } else {
                                            depot = depots[idDepot].depot;
                                        }

                                        // On regarde si on a une image.
                                        if (idPhoto) {
                                            var photo = depot.addPhoto();
                                            photo.id = idPhoto;
                                            photo.nativeUrl = result.native_url;
                                            photo.isSync = result.picture_is_sync == "true" ? true : false;

                                            depots[idDepot].photosAutoincrementMax = Math.max(depots[idDepot].photosAutoincrementMax, idPhoto);
                                        }
                                    }
                                }

                                for (var idDepot in depots) {
                                    // On met à jour les ids max de photos pour chaque depots.
                                    depots[idDepot].depot.photosAutoincrement = depots[idDepot].photosAutoincrementMax;
                                }

                                var projetsToInsert = [];
                                for (var idProjet in projets) {
                                    // On met à jour les ids max de depots pour chaque projet et on l'enregistre.
                                    projets[idProjet].projet.depotsAutoincrement = projets[idProjet].depotsAutoincrementMax;
                                    projetsToInsert.push(projets[idProjet].projet);
                                }

                                // On enregistre les projets.
                                var projetsInsertPromise;
                                if (projetsToInsert.length > 0) {
                                    projetsInsertPromise = that.IndexedDBService.insertProjet(projetsToInsert);
                                } else {
                                    projetsInsertPromise = Promise.resolve();
                                }
                                projetsInsertPromise.then(function () {
                                    console.log("Les projets ont été insérés dans IndexedDB.");

                                    // On va récupérer les configs.
                                    var request = that.GeneanetDatabase.createRequest("SELECT * FROM configuration");
                                    that.GeneanetDatabase.executeRequest(request, function(tx, results) {
                                        for (var i = 0; i < results.rows.length; i++) {
                                            var result = results.rows[i];
                                            var valueConfiguration = result.value;
                                            valueConfiguration = valueConfiguration === 'true' ? true : valueConfiguration;
                                            valueConfiguration = valueConfiguration === 'false' ? false : valueConfiguration;
                                            that.ConfigurationService.add(result.key, valueConfiguration);
                                        }

                                        // On supprime les tables.
                                        var deleteTable = function (tableName) {
                                            return new Promise(function (resolve) {
                                                var request = that.GeneanetDatabase.createRequest('DROP TABLE '+tableName);
                                                request.setIsReadAndWrite();
                                                that.GeneanetDatabase.executeRequest(request, function(tx, results) {
                                                    console.log("Table "+tableName+" supprimée.");
                                                    resolve();
                                                });
                                            });
                                        };
                                        var tables = ['configuration', 'photo', 'depot', 'projet'];
                                        var promiseTablesDeleted = [];
                                        for (var i = 0; i < tables.length; i++) {
                                            promiseTablesDeleted.push(deleteTable(tables[i]));
                                        }
                                        Promise.all(promiseTablesDeleted).then(function () {
                                            console.log("Toutes les tables ont été supprimées.")
                                            that.GeneanetLocalStorage.setItem('webSqlToIndexedDB', true, 'migration');

                                            resolve();
                                        });
                                    });
                                }).catch(function () {
                                    console.error("Les projets n'ont pu être insérés dans IndexedDB.");
                                });
                            });
                        } else {
                            // Les tables n'existent pas, on a rien a migrer.
                            that.GeneanetLocalStorage.setItem('webSqlToIndexedDB', true, 'migration');
                            resolve();
                        }
                    });
                } else {
                    // La migration a déjà eu lieu.
                    resolve();
                }
            });
        });
    };

    return [
        'GeneanetLocalStorage',
        'IndexedDBService',
        'GeneanetDatabase',
        'ConfigurationService',
        MigrationService
    ];
})();
