'use strict';

/**
 * Service spécifique à l'application Sauvons nos tombes
 * Permet de gérer gérant les projets, depots et photos.
 **/
var projetService = (function () {
    /**
     * Service gérant les projets, dépôts et photos.
     *
     * @param {Object} $rootScope
     * @param {Object} $translate
     * @param {Object} $route
     * @param {Object} GeneanetOverlay
     * @param {Object} GeneanetCamera
     * @param {Object} GeneanetDevice
     * @param {Object} GeneanetFile
     * @param {Object} GeneanetDate
     * @param {Object} GeneanetGeolocation
     * @param {IndexedDBService} IndexedDBService
     * @param {Object} MigrationService
     *
     * @constructor
     */
    function ProjetService($rootScope, $translate, $route, GeneanetOverlay, GeneanetCamera, GeneanetDevice, GeneanetFile, GeneanetDate, GeneanetGeolocation, IndexedDBService, MigrationService) {
        this.$rootScope = $rootScope;
        this.$translate = $translate;
        this.$route = $route;
        this.GeneanetOverlay = GeneanetOverlay;
        this.GeneanetCamera = GeneanetCamera;
        this.GeneanetDevice = GeneanetDevice;
        this.GeneanetFile = GeneanetFile;
        this.GeneanetDate = GeneanetDate;
        this.GeneanetGeolocation = GeneanetGeolocation;
        this.IndexedDBService = IndexedDBService;
        this.MigrationService = MigrationService;

        // Permettent de suivre le chargement des projets.
        this.projets = null;
        this.promiseToLoadProjets = null;

        // Permettent de suivre la navigation dans l'application (projet, depot, photo courant(e)).
        this.currentProjet = null;
        this.currentDepot = null;
        this.currentPhoto = null;
    }

    /**
     * Charge l'ensemble des projets si pas déjà fait.
     *
     * @return {Promise}
     */
    ProjetService.prototype.ensureProjetsLoaded = function () {
        if (this.promiseToLoadProjets) {
            return this.promiseToLoadProjets;
        }

        var that = this;
        this.promiseToLoadProjets = new Promise(function (resolve) {
            that.GeneanetOverlay.openSpinner(that.$translate.instant('spinner.get_data'));

            that.MigrationService.migrate().then(function () {
                that.IndexedDBService.getAllProjets().then(function (projets) {
                    that.projets = projets;
                    that.GeneanetOverlay.close();

                    resolve();
                });
            });
        });

        return this.promiseToLoadProjets;
    };

    /**
     * Reset le contexte de navigation.
     */
    ProjetService.prototype.resetContext = function () {
        this.currentProjet = null;
        this.currentDepot = null;
        this.currentPhoto = null;
    };

    /**
     * Redirige vers la page gallery en passant par la homepage et en conservant le projet courant.
     */
    ProjetService.prototype.redirectToGallery = function () {
        // On "repasse" par la home.
        history.pushState(null, 'home', '#');
        // Puis on passe sur la gallery.
        history.pushState(null, 'gallery', '#/galleryProject/true');
        // Et on reload pour que angular actualise par rapport à la vue courante.
        this.$route.reload();
    };

    /**
     * Insère un nouveau projet.
     *
     * @param {Projet} projet
     *
     * @return {Promise}
     */
    ProjetService.prototype.insertProjet = function (projet) {
        this.projets.unshift(projet);

        return this.IndexedDBService.insertProjet(projet);
    };

    /**
     * Supprime le projet.
     *
     * @param {Projet} projet
     *
     * @return {Promise}
     */
    ProjetService.prototype.deleteProjet = function (projet) {
        var position = this.projets.indexOf(projet);
        if (position != -1) {
            this.projets.splice(position, 1);
        }

        return this.IndexedDBService.deleteProjetFromId(projet.id);
    };

    /**
     * Met à jour le projet.
     * Simple proxy pour limiter les dépendances.
     *
     * @param {Projet} projet
     *
     * @return {Promise}
     */
    ProjetService.prototype.updateProjet = function (projet) {
        return this.IndexedDBService.updateProjet(projet);
    };

    /**
     * Ajoute une nouvelle photo au projet courant.
     *
     * @param {integer|null} sectionId
     *
     * @return {Promise}
     */
    ProjetService.prototype.addPhoto = function (sectionId) {
        var that = this;

        return new Promise(function (resolve, reject) {
            if (that.currentProjet) {
                that.takePhoto()
                    .then(function (photoUrl) {
                        // La photo est prise, à partir de son url, on va chercher les entries nécessaires pour la déplacer.
                        return that.getPhotoAndDirEntriesFromUrl(photoUrl);
                    })
                    .then(function (data) {
                        // On a les entries nécessaires, on déplace la photo là où il faut.
                        return that.movePhoto(data.photoFileEntry, data.entityDestination);
                    })
                    .then(function (photoFileEntry) {
                        // La photo a été déplacée, on va mettre à jour le projet et enregistrer.
                        var nativeUrl = photoFileEntry.nativeURL;

                        if (that.currentProjet.isPrivate() && that.currentProjet.depotsAutoincrement == 1) {
                            // On regarde si le projet est privé et que c'est sa première photo pour changer/corriger la localisation du projet.
                            that.updateProjetWithCurrentPosition(that.currentProjet);
                        }

                        if (!that.currentDepot) {
                            that.currentDepot = that.currentProjet.addDepot();
                            that.currentDepot.sectionId = sectionId;
                        }
                        that.currentPhoto = that.currentDepot.addPhoto();
                        that.currentPhoto.nativeUrl = photoFileEntry.nativeURL;

                        // Tout est prêt, on met à jour le projet.
                        that.IndexedDBService.updateProjet(that.currentProjet).then(function () {
                            resolve(that.currentPhoto);
                        });
                    })
                    .catch(function () {
                        reject("Erreur lors de la prise de photo.");
                    });
            } else {
                reject("Il faut choisir un projet pour faire une nouvelle photo.");
            }
        });
    };

    /**
     * Démarre la prise de photo.
     *
     * @return {Promise}
     */
    ProjetService.prototype.takePhoto = function () {
        var that = this;

        return new Promise(function (resolve, reject) {
            that.GeneanetCamera.prendreUnePhoto(
                function (photoUrl) {
                    resolve(photoUrl);
                },
                function () {
                    console.error("Erreur pour la prise de photo");
                    console.log(arguments);
                    reject();
                },
                false
            );
        });
    };

    /**
     * Récupère une entry représentant le fichier pointé par photoUrl et une entry vers le dossier où l'on souhaite stocker les photos.
     *
     * @param {String} photoUrl
     *
     * @return {Promise}
     */
    ProjetService.prototype.getPhotoAndDirEntriesFromUrl = function (photoUrl) {
        var that = this;

        return new Promise(function (resolve, reject) {
            window.resolveLocalFileSystemURL(
                photoUrl,
                function (fileEntry) {
                    // On va récupérer le dossier vers lequel on veut déplacer le fichier.
                    if (that.GeneanetDevice.isAndroidPlatform()) {
                        // Dans le cas d'android, on demande un accès au système de fichier pour aller
                        // stocker le fichier dans Pictures/NotreDossier
                        window.requestFileSystem(
                            LocalFileSystem.PERSISTENT,
                            0,
                            function (fs) {
                                fs.root.getDirectory(
                                    'Pictures/'+that.GeneanetDevice.getNameDirectory(),
                                    {create: true},
                                    function (dirEntry) {
                                        // Maintenant que l'on a le dossier de destination, on le transmet.
                                        resolve({photoFileEntry: fileEntry, entityDestination: dirEntry});
                                    },
                                    function () {
                                        console.error('Erreur dans la récupération du dossier de destination');
                                        console.log(arguments);
                                        reject();
                                    }
                                );
                            },
                            function () {
                                console.error('Erreur lors de la récupération du système de fichier.');
                                console.log(arguments);
                                reject();
                            }
                        );
                    } else {
                        that.GeneanetFile.getFileFromUrl(cordova.file.syncedDataDirectory).then(function (entityDestination) {
                            resolve({photoFileEntry: fileEntry, entityDestination: entityDestination});
                        });
                    }
                },
                function (codeError) {
                    console.error("Erreur pour résoudre l'uri de la photo ("+photoUrl+"), code '"+codeError.code+"'");
                    reject();
                }
            );
        });
    };

    /**
     * Déplace un fichier vers le dossier désiré en le renommant pour avoir un nom maitrisé.
     *
     * @param {Object} photoFileEntry
     * @param {Object} dirEntry
     *
     * @return {Promise}
     */
    ProjetService.prototype.movePhoto = function (photoFileEntry, dirEntry) {
        var nameFileDestination = this.generateNameImage(this.GeneanetFile.getExtensionFile(photoFileEntry.name));

        return new Promise(function (resolve, reject) {
            photoFileEntry.moveTo(
                dirEntry,
                nameFileDestination,
                function (newPhotoFileEntry) {
                    resolve(newPhotoFileEntry);
                },
                function (codeError) {
                    reject("Erreur pour déplacer le fichier, code : '"+codeError+"'");
                }
            );
        });
    };

    /**
     * Permet de générer un nom d'image depuis la date courrante.
     *
     * @param {string} extension Nom de l'extention de fichier.
     *
     * @return {string} Renvoie le nom générer.
     */
    ProjetService.prototype.generateNameImage = function (extension) {
        var elementsToGenerateName = [];
        elementsToGenerateName.push("GNN");

        var currentDate = new Date();
        var year = String(currentDate.getFullYear());
        var month = String(this.GeneanetDate.formatedTime(currentDate.getMonth()+1));
        var day = String(this.GeneanetDate.formatedTime(currentDate.getDate()));
        var hour = String(this.GeneanetDate.formatedTime(currentDate.getHours()));
        var minute = String(this.GeneanetDate.formatedTime(currentDate.getMinutes()));
        var second = String(this.GeneanetDate.formatedTime(currentDate.getSeconds()));

        elementsToGenerateName.push(year+month+day);
        elementsToGenerateName.push(hour+minute+second);

        return elementsToGenerateName.join("_") + "." + extension;
    };

    /**
     * Met à jour la position du projet avec la position actuelle si possible.
     *
     * @param {Projet} projet
     *
     * @return {Promise}
     */
    ProjetService.prototype.updateProjetWithCurrentPosition = function (projet) {
        var that = this;

        return new Promise(function (resolve, reject) {
            that.GeneanetGeolocation.getCurrentPosition()
                .then(function (position) {
                    projet.setGeo([position.coords.latitude, position.coords.longitude]);
                    projet.isSync = false;
                    that.IndexedDBService.updateProjet(projet).then(function () {
                        // On met à jour le projet.
                        resolve();
                    }).catch(function () {
                        reject("Le projet n'a pu être mis à jour.");
                    });
                })
                .catch(function () {
                    reject("Erreur pour optenir la localisation.");
                });
        });
    };

    return [
        '$rootScope',
        '$translate',
        '$route',
        'GeneanetOverlay',
        'GeneanetCamera',
        'GeneanetDevice',
        'GeneanetFile',
        'GeneanetDate',
        'GeneanetGeolocation',
        'IndexedDBService',
        'MigrationService',
        ProjetService
    ];
})();
