(function () {
    'use strict';

    /**
     * Permet de suivre le déroulement de plusieurs actions et ainsi les stopper.
     *
     * @constructor
     */
    function GeneanetActionsManager() {
        this.actions = [];
    }

    /**
     * Génère une nouvelle action
     *
     * @param {mixed} data Permettant de faire transiter des infos vis à vis d'une action.
     *
     * @return {Action}
     */
    GeneanetActionsManager.prototype.createAction = function (data) {
        var action = new Action(data);
        action.start();
        this.actions.push(action);

        return action;
    };

    /**
     * Stoppe toutes les actions en cours.
     *
     * @return {Promise} Résolue quand tout est stoppé.
     */
    GeneanetActionsManager.prototype.stopAll = function () {
        var promises = [];

        for (var i = 0; i < this.actions.length; i++) {
            var action = this.actions[i];
            if (!action.inProgress()) {
                continue;
            }
            action.askToStop();
            promises.push(action.promise);
        }

        return new Promise(function (resolve, reject) {
            Promise.all(promises).finally(function () {
                resolve();
            });
        });
    };

    /**
     * Permet de suivre le déroulement d'une action et la stopper.
     *
     * @param {mixed} data Permettant de faire transiter des infos vis à vis d'une action.
     *
     * @constructor
     */
    function Action(data) {
        this.status = Action.STATUS_DEFAULT;
        this.resolve = null;
        this.reject = null;
        this.data = data;

        var that = this;
        this.promise = new Promise(function (resolve, reject) {
            that.resolve = resolve;
            that.reject = reject;
        });
    }
    Action.STATUS_DEFAULT = 0;
    Action.STATUS_STARTED = 1;
    Action.STATUS_DONE = 2;
    Action.STATUS_WAIT_TO_STOP = -1;
    Action.STATUS_STOPPED = -2;

    /**
     * Démarre une action.
     */
    Action.prototype.start = function () {
        this.status = Action.STATUS_STARTED;
    };

    /**
     * Indique qu'une action est terminée.
     *
     * @return {Promise}
     */
    Action.prototype.done = function () {
        this.status = Action.STATUS_DONE;
        this.resolve();

        return this.promise;
    };

    /**
     * Demande à stopper cette action.
     */
    Action.prototype.askToStop = function () {
        this.status = Action.STATUS_WAIT_TO_STOP;
    };

    /**
     * Arrête l'action.
     *
     * @return {Promise}
     */
    Action.prototype.stop = function () {
        this.status = Action.STATUS_STOPPED;
        this.reject();

        return this.promise;
    };

    /**
     * Regarde si l'on peut continuer ou non. Si on nous a demandé de stopper, la réponse est non et on déclenche l'arrêt de l'action.
     *
     * @return {Boolean}
     */
    Action.prototype.canContinue = function () {
        if (this.status == Action.STATUS_STARTED) {
            return true;
        } else if (this.status == Action.STATUS_WAIT_TO_STOP) {
            this.stop();
        }

        return false;
    };

    /**
     * Indique si une action est en train de tourner.
     *
     * @return {Boolean}
     */
    Action.prototype.inProgress = function () {
        return this.status == Action.STATUS_STARTED;
    };

    window.GeneanetActionsManager = window.GeneanetActionsManager || GeneanetActionsManager;
})();