(function () {
    'use strict';

    // Les modes autorisés pour ouvrir une transaction.
    var TRANSACTION_MODE = {
        READONLY: 'readonly',
        READWRITE: 'readwrite',
        VERSIONCHANGE: 'versionchange'
    };

    // Les directions autorisées lors d'une requête.
    var CURSOR_DIRECTION = {
        NEXT: 'next',
        NEXTUNIQUE: 'nextunique',
        PREV: 'prev',
        PREVUNIQUE: 'prevunique'
    };

    /**
     * Objet permettant de manipuler les objets dans IndexedDB.
     *
     * @param {String} dbName
     * @param {Integer} dbVersion
     *
     * @constructor
     */
    function GeneanetIndexedDB(dbName, dbVersion) {
        this.dbName = dbName;
        this.dbVersion = dbVersion;
        this.entities = {};
        this.callbackOnversionchange = null;

        this.db = null;
    }
    GeneanetIndexedDB.CURSOR_DIRECTION = CURSOR_DIRECTION;

    /**
     * Permet de renseigner une callback dans le cas où la connexion a du être coupée car un autre onglet à mis à jour le modèle des données.
     *
     * @param {Function} callbackOnversionchange
     */
    GeneanetIndexedDB.prototype.setCallbackOnversionchange = function (callbackOnversionchange) {
        this.callbackOnversionchange = callbackOnversionchange;
    };

    /**
     * Enregistre une entitée.
     *
     * @param {Function} constructor Permet de construire l'objet associé à l'entrée dans le stockage.
     * @param {String} objectStoreName Nom de l'objectStore.
     * @param {Object} config Config de l'objectStore
     * @param {Object|undefined} indexes Liste des indexes a créer.
     */
    GeneanetIndexedDB.prototype.registerEntity = function (constructor, objectStoreName, config, indexes) {
        this.entities[objectStoreName] = {
            constructor: constructor,
            config: config,
            indexes: indexes || [],
        };
    };

    /**
     * Démarre la connexion. Les callback récupèrent en argument la connexion.
     *
     * @param {Function} preCallback Appelée avant la génération des objectStore.
     * @param {Function} postCallback Appelée après la génération des objectStore.
     *
     * @return {Promise}
     */
    GeneanetIndexedDB.prototype.initConnexion = function (preCallback, postCallback) {
        var that = this;

        return new Promise(function (resolve, reject) {
            // On démarre la connexion.
            var request = window.indexedDB.open(that.dbName, that.dbVersion);
            request.onerror = function (event) {
                console.error("Erreur lors de la connexion.");
                console.error(event);
                reject();
            };

            request.onblocked = function (event) {
                console.log("Ouverture bloquée car un autre onglet est ouvert avec une version plus ancienne.");
            };

            request.onsuccess = function (event) {
                that.db = event.target.result;

                // On greffe une callback globale pour les erreurs.
                that.db.onerror = function (event) {
                    console.error("Erreur avec indexeddb : " + event.target.errorCode);
                    console.error(event);
                };

                that.db.onversionchange = function (event) {
                    console.log("Un autre onglet a ouvert une connexion avec une version plus récente.");
                    that.db.close();
                    if (that.callbackOnversionchange) {
                        that.callbackOnversionchange();
                    }
                };

                resolve(that.db);
            };

            request.onupgradeneeded = function(event) {
                var db = event.target.result;

                if (preCallback) {
                    preCallback(db);
                }

                for (var objectStoreName in that.entities) {
                    var objectStore;
                    if (!db.objectStoreNames.contains(objectStoreName)) {
                        // Crée un objet de stockage pour cette base de donnée si non existant.
                        objectStore = db.createObjectStore(objectStoreName, that.entities[objectStoreName].config);
                    } else {
                        objectStore = that.extratObjectStores(request.transaction, objectStoreName);
                    }
                    if (that.entities[objectStoreName].indexes.length) {
                        for (var i = 0; i < that.entities[objectStoreName].indexes.length; i++) {
                            var index = that.entities[objectStoreName].indexes[i];
                            if (!objectStore.indexNames.contains(index[0])) {
                                objectStore.createIndex(index[0], index[1]);
                            }
                        }
                    }
                }

                if (postCallback) {
                    postCallback(db);
                }
            };
        });
    };

    /**
     * Permet de stocker un ou des objets.
     *
     * @param {GeneanetIndexedDBEntity|Array} data
     *
     * @return {Promise}
     */
    GeneanetIndexedDB.prototype.insert = function (data) {
        var that = this;
        var objectStoreNames = this.extractObjectStoreNamesFromData(data);

        return new Promise(function (resolve, reject) {
            var transaction = that.createTransaction(objectStoreNames, TRANSACTION_MODE.READWRITE);
            transaction.oncomplete = function (event) {
                resolve();
            };
            transaction.onerror = function (event) {
                console.error("Erreur dans la transaction : "+event.target.error);
                event.stopPropagation();
                reject();
            };

            // On passe par une méthode pour ajouter la callback sur onsuccess pour conserver la bonne entitée.
            var addCallbackOnsuccess = function (request, entity) {
                request.onsuccess = function (event) {
                    var objectStoreConfig = that.entities[entity.getObjectStoreName()].config;
                    if (objectStoreConfig.autoIncrement) {
                        entity[objectStoreConfig.keyPath] = event.target.result;
                    }
                };
            };

            var objectStores = that.extratObjectStores(transaction, objectStoreNames);
            if (data instanceof GeneanetIndexedDBEntity) {
                data = [data];
            }
            for (var i = 0; i < data.length; i++) {
                try {
                    var request = objectStores[data[i].getObjectStoreName()].add(data[i]);
                    addCallbackOnsuccess(request, data[i]);
                } catch (e) {
                    console.error("Erreur lors de l'insert, on abort la transaction : "+e.message);
                    transaction.abort();
                    reject();
                    break;
                }
            }
        });
    };

    /**
     * Permet de mettre à jour un ou des objets.
     *
     * @param {GeneanetIndexedDBEntity|Array} data
     *
     * @return {Promise}
     */
    GeneanetIndexedDB.prototype.update = function (data) {
        var that = this;
        var objectStoreNames = this.extractObjectStoreNamesFromData(data);

        return new Promise(function (resolve, reject) {
            var transaction = that.createTransaction(objectStoreNames, TRANSACTION_MODE.READWRITE);
            transaction.oncomplete = function (event) {
                resolve();
            };
            transaction.onerror = function (event) {
                console.error("Erreur dans la transaction : "+event.target.error);
                event.stopPropagation();
                reject();
            };

            var objectStores = that.extratObjectStores(transaction, objectStoreNames);
            if (data instanceof GeneanetIndexedDBEntity) {
                data = [data];
            }
            for (var i = 0; i < data.length; i++) {
                try {
                    var request = objectStores[data[i].getObjectStoreName()].put(data[i]);
                } catch (e) {
                    console.error("Erreur lors de la mise à jour, on abort la transaction : "+e.message);
                    transaction.abort();
                    reject();
                    break;
                }
            }
        });
    };

    /**
     * Permet de supprimer un ou des objets via leur ids.
     *
     * @param {Object} data Un objet avec en clé les objectStoreName => array d'ids.
     *
     * @return {Promise}
     */
    GeneanetIndexedDB.prototype.delete = function (data) {
        var that = this;
        var objectStoreNames = Object.keys(data);

        return new Promise(function (resolve, reject) {
            var transaction = that.createTransaction(objectStoreNames, TRANSACTION_MODE.READWRITE);
            transaction.oncomplete = function (event) {
                resolve();
            };
            transaction.onerror = function (event) {
                console.error("Erreur dans la transaction : "+event.target.error);
                event.stopPropagation();
                reject();
            };

            var objectStores = that.extratObjectStores(transaction, objectStoreNames);
            for (var objectStoreName in data) {
                for (var i = 0; i < data[objectStoreName].length; i++) {
                    try {
                        objectStores[objectStoreName].delete(data[objectStoreName][i]);
                    } catch (e) {
                        console.error("Erreur lors du delete, on abort la transaction : "+e.message);
                        transaction.abort();
                        reject();
                        break;
                    }
                }
            }
        });
    };

    /**
     * Permet de récupérer un ou des objets via leur ids.
     *
     * @param {String} objectStoreName
     * @param {Array} ids
     *
     * @return {Promise}
     */
    GeneanetIndexedDB.prototype.getByIds = function (objectStoreName, ids) {
        var that = this;

        return new Promise(function (resolve, reject) {
            var results = {};

            var transaction = that.createTransaction(objectStoreName, TRANSACTION_MODE.READONLY);
            transaction.oncomplete = function (event) {
                resolve(results);
            };
            transaction.onerror = function (event) {
                console.error("Erreur dans la transaction : "+event.target.error);
                event.stopPropagation();
                reject();
            };

            // On passe par une méthode pour ajouter la callback sur onsuccess pour conserver le bon id durant la boucle.
            var addCallbackOnsuccess = function (request, id) {
                request.onsuccess = function (event) {
                    var result = event.target.result;
                    if (result === undefined) {
                        return;
                    }
                    if (that.entities[objectStoreName]) {
                        result = new that.entities[objectStoreName].constructor(result);
                    }
                    results[id] = result;
                };
            };
            var callbackError = function (event) {
                console.error("Erreur lors de la récupération : "+event.target.error);
            };

            var objectStore = that.extratObjectStores(transaction, objectStoreName);
            for (var i = 0; i < ids.length; i++) {
                try {
                    var request = objectStore.get(ids[i]);
                    request.onerror = callbackError;
                    addCallbackOnsuccess(request, ids[i]);
                } catch (e) {
                    console.error("Erreur lors de la récupération, on abort la transaction : "+e.message);
                    transaction.abort();
                    reject();
                    break;
                }
            }
        });
    };

    /**
     * Permet de récupérer tous les objets stockés dans l'objectStore demandé.
     *
     * @param {String} objectStoreName
     * @param {String|null} direction Indique le sens de parcours (voir GeneanetIndexedDB.CURSOR_DIRECTION / https://developer.mozilla.org/fr/docs/Web/API/IDBCursor/direction)
     * @param {IDBKeyRange|null} range Permet de filtrer les éléments récupérés (voir https://developer.mozilla.org/fr/docs/Web/API/IDBKeyRange).
     * @param {String|null} indexName Permet de rechercher via un index plutôt que l'id.
     *
     * @return {Promise}
     */
    GeneanetIndexedDB.prototype.getAll = function (objectStoreName, direction, range, indexName) {
        var that = this;

        direction = direction || undefined;
        range = range || null;

        return new Promise(function (resolve, reject) {
            var results = [];

            var transaction = that.createTransaction(objectStoreName, TRANSACTION_MODE.READONLY);
            transaction.oncomplete = function (event) {
                resolve(results);
            };
            transaction.onerror = function (event) {
                console.error("Erreur dans la transaction : "+event.target.error);
                event.stopPropagation();
                reject();
            };

            var callbackOnsuccess = function (event) {
                var cursor = event.target.result;

                if (cursor) {
                    var result = cursor.value;
                    if (that.entities[objectStoreName]) {
                        result = new that.entities[objectStoreName].constructor(result);
                    }
                    results.push(result);
                    cursor.continue();
                }
            };
            var callbackError = function (event) {
                console.error("Erreur lors de la récupération : "+event.target.error);
            };

            var objectStore = that.extratObjectStores(transaction, objectStoreName);
            try {
                var targetToOpenCursor = indexName ? objectStore.index(indexName) : objectStore;
                var cursor = targetToOpenCursor.openCursor(range, direction);
                cursor.onsuccess = callbackOnsuccess;
                cursor.onerror = callbackError;
            } catch (e) {
                console.error("Erreur lors de la récupération des données, on abort la transaction : "+e.message);
                transaction.abort();
                reject();
            }
        });
    };

    /**
     * Extrait de l'objet ou des objets les objectStore concernés.
     *
     * @param {GeneanetIndexedDBEntity|Array} data
     *
     * @return {Array}
     *
     * @throw Error si data n'est pas bon.
     */
    GeneanetIndexedDB.prototype.extractObjectStoreNamesFromData = function (data) {
        var objectStoreNames = [];
        if (data instanceof GeneanetIndexedDBEntity) {
            objectStoreNames.push(data.getObjectStoreName());
        } else if (data instanceof Array) {
            for (var i = 0; i < data.length; i++) {
                objectStoreNames.push(data[i].getObjectStoreName());
            }
        } else {
            throw Error("La donnée doit être une instance de GeneanetIndexedDBEntity ou un array.");
        }

        return objectStoreNames;
    };

    /**
     * Récupère depuis la transaction les objectStores demandés.
     *
     * @param {IDBTransaction} transaction
     * @param {Array|String} objectStoreNames
     *
     * @return {Object} Une liste des objectStore ou l'objectStore directement si un seul demandé.
     */
    GeneanetIndexedDB.prototype.extratObjectStores = function (transaction, objectStoreNames) {
        if (objectStoreNames instanceof Array) {
            var objectStores = {};
            for (var i = 0; i < objectStoreNames.length; i++) {
                objectStores[objectStoreNames[i]] = transaction.objectStore(objectStoreNames[i]);
            }

            return objectStores;
        } else {
            return transaction.objectStore(objectStoreNames);
        }
    };

    /**
     * Génère une transaction.
     *
     * @param {Array} objectStoreNames Les différents objectStore que l'on veut manipuler.
     * @param {String} mode Les modes d'accès (readonly, readwrite ou versionchange).
     *
     * @return {IDBTransaction}
     */
    GeneanetIndexedDB.prototype.createTransaction = function (objectStoreNames, mode) {
        mode = mode || TRANSACTION_MODE.READONLY;

        return this.db.transaction(objectStoreNames, mode);
    };

    /**
     * Les objets héritant de ce constructor indique le type d'objectStore associé et indique qu'ils peuvent être stockés.
     *
     * @constructor
     */
    function GeneanetIndexedDBEntity() {
    }

    /**
     * Méthode a implémenter permettant de retourner l'objectStoreName associé à l'entitée.
     *
     * @return {String}
     */
    GeneanetIndexedDBEntity.prototype.getObjectStoreName = function () {
        throw Error("Vous devez implémenter getObjectStoreName dans votre objet "+this.constructor.name);
    };

    /**
     * Méthode a appeler au construct permettant de compléter l'entitée depuis l'objet provenant du système de stockage.
     *
     * @param {Object} data
     */
    GeneanetIndexedDBEntity.prototype.populateFromStorage = function (data) {
        if (data instanceof Object) {
            Object.assign(this, data);
        }
    };

    window.GeneanetIndexedDB = window.GeneanetIndexedDB || GeneanetIndexedDB;
    window.GeneanetIndexedDBEntity = window.GeneanetIndexedDBEntity || GeneanetIndexedDBEntity;
})();
